//+------------------------------------------------------------------+
//|                                                   ChartEvent.mq5 |
//|                                            Copyright 2020, ernst |
//|                             https://www.mql5.com/en/users/pippod |
//+------------------------------------------------------------------+
#property copyright "Copyright 2020, ernst"
#property link      "https://www.mql5.com/en/users/pippod"
#property version   "1.00"
#property indicator_chart_window
//---
input string TickName="Tick";
uint tickCount;
//+------------------------------------------------------------------+
//| Tick chart(s) operations class                                   |
//+------------------------------------------------------------------+
#include <ChartObjects\ChartObjectsTxtControls.mqh>
class COnTick : public CChartObjectButton
  {
private:
   long              m_chartID;
   string            m_indname;
   string            m_symbol[];
   ENUM_TIMEFRAMES   m_timeframe;
public:
//---                constructor
                     COnTick(void)  {}
//---                destructor
                    ~COnTick(void)  {}
//---
   bool              OnInit(const string name,const string &symbol[],ENUM_TIMEFRAMES timeframe);
//---
   bool              OnTick(const string &symbol);
//---
   bool              OnDeinit(void);
//---
   void              OnChartEvent(const string &sparam);
//---
   bool              ChartOpen(void);
//---
   void              ChartClose(void);
//---
  }*btn;       
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COnTick::OnInit(const string name,const string &symbol_name[],ENUM_TIMEFRAMES timeframe)
  {
//---
   m_chartID=ChartID();
   m_indname=name;
   m_timeframe=timeframe;
   int size=ArraySize(symbol_name);
   ArrayResize(m_symbol,size);
   for(int i=0;i<size;i++)
      if(!SymbolSelect(m_symbol[i]=symbol_name[i],true))
         return(false);
//--- create button for tick charts
   if(Create(m_chartID,"symbols",ChartWindowFind(),12,42,150,30))
     {
      State(false);
      Color(clrFireBrick);
      BackColor(clrLimeGreen);
      SetString(OBJPROP_TEXT," ");
      Background(false);
      SetInteger(OBJPROP_FONTSIZE,15);
      SetString(OBJPROP_FONT,"Arial Bold");
      Corner(CORNER_LEFT_LOWER);
     }
//---
   printf("OnTick inititalized. Chart ID: %I64d",m_chartID);
//---
   return(true);
  }        
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COnTick::OnTick(const string &symbol_name)
  {
   SetString(OBJPROP_TEXT,symbol_name);
   return(true);
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COnTick::OnDeinit(void)
  {
//---
   Delete();
   this.ChartClose();
//---
   return(true);
  }      
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void COnTick::OnChartEvent(const string &sparam)
  {
//--- check button clicked/state
   if(sparam==Name())
     {
      if(!State())
        {
         Color(clrFireBrick);
         BackColor(clrLimeGreen);
         SetString(OBJPROP_TEXT," ");
         this.ChartClose();
        }
      else
        {
         Color(clrLimeGreen);
         BackColor(clrFireBrick);
         this.ChartOpen();
        }
     }
//---
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COnTick::ChartOpen(void)
  {
   long chartId; 
//--- open tick chart(s) for each symbol
   for(int i=0;i<ArraySize(m_symbol);i++)
     { 
      bool chartFound=false;
//---
      chartId=ChartFirst(); 
      while(chartId>-1)
        { 
         if(ChartSymbol(chartId)==this.m_symbol[i])
            for(int m=0;m<ChartIndicatorsTotal(chartId,0);m++)
               if(ChartIndicatorName(chartId,0,m)==m_indname)
                 { 
                  chartFound=true;
                  break;
                 }
         if(chartFound || (chartId=ChartNext(chartId))<0) 
            break; 
        } 
//---
      if(!chartFound)
        {
         if(!ChartApplyTemplate(ChartOpen(this.m_symbol[i],m_timeframe),this.m_indname))   
           { 
            printf("Chart Template %s.tpl not found",this.m_indname);
            return(false);
           }
         //iCustom(m_symbol[i].Symbol(),_Period,name,ChartID(),index,0,0);
        }    
     }
//---
   return(true);
  }   
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void COnTick::ChartClose(void)
  {
   long chartId=ChartFirst(); 
//--- close tick chart(s)
   while(chartId>-1)
     { 
      for(int m=0;m<ChartIndicatorsTotal(chartId,0);m++)
         if(ChartIndicatorName(chartId,0,m)==this.m_indname)
           { 
            ::ChartClose(chartId);
            break;
           }
      chartId=ChartNext(chartId);
     } 
//---
  }
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- indicator buffers mapping
   IndicatorSetString(INDICATOR_SHORTNAME,"ChartEvent");
   long chartID=ChartID();
   ChartSetInteger(chartID,CHART_SCALE,4);
   ChartSetInteger(chartID,CHART_MODE,CHART_CANDLES);
//--- indicator symbols
   string Symbol[]={"AUDUSD","USDCAD","USDCHF","EURUSD","EURJPY","GBPUSD","GBPJPY","USDJPY","NZDUSD"};
   if(!CheckPointer(btn=new COnTick) || !btn.OnInit(TickName,Symbol,(ENUM_TIMEFRAMES)_Period))
      return(INIT_FAILED);
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   if(CheckPointer(btn))
     {
      btn.OnDeinit(); 
      delete btn;
     }
//---
  }   
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const int begin,
                const double &price[])
  {
//---
   tickCount=GetTickCount();
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| ChartEvent function                                              |
//+------------------------------------------------------------------+
void OnChartEvent(const int id,
                  const long &lparam,
                  const double &dparam,
                  const string &sparam)
  {
//---
   if(id>CHARTEVENT_CUSTOM && btn!=NULL)
      btn.SetString(OBJPROP_TEXT,StringFormat("%s %d",sparam,lparam-tickCount));
//---
   if(id==CHARTEVENT_OBJECT_CLICK && btn!=NULL)
      btn.OnChartEvent(sparam);
  }
//+------------------------------------------------------------------+
